@@ -26,8 +26,7 @@
 
 #include "tls_internal.h"
 
-struct tls *
-tls_server(void)
+struct tls *tls_server(void)
 {
 	struct tls *ctx;
 
@@ -39,8 +38,7 @@ tls_server(void)
 	return (ctx);
 }
 
-struct tls *
-tls_server_conn(struct tls *ctx)
+struct tls *tls_server_conn(struct tls *ctx)
 {
 	struct tls *conn_ctx;
 
@@ -52,8 +50,48 @@ tls_server_conn(struct tls *ctx)
 	return (conn_ctx);
 }
 
-int
-tls_configure_server(struct tls *ctx)
+#define PG_ALPN_PROTOCOL_VECTOR { 10, 'p', 'o', 's', 't', 'g', 'r', 'e', 's', 'q', 'l' }
+static const unsigned char alpn_protos[] = PG_ALPN_PROTOCOL_VECTOR;
+
+/*
+ * Server callback for ALPN negotiation.
+ *
+ * This callback is based on
+ * https://github.com/postgres/postgres/blob/b5c53b403c93393c3725558294cbf4dbfb575e42/src/backend/libpq/be-secure-openssl.c#L1327-L1370
+ *
+ * The only difference is userdata is not passed in by the caller (it's
+ * unused here).
+ *
+ * c.f. https://www.openssl.org/docs/manmaster/man3/SSL_CTX_set_alpn_select_cb.html
+ */
+static int alpn_cb(SSL *ssl, const unsigned char **out, unsigned char *outlen,
+		   const unsigned char *in, unsigned int inlen, void *userdata)
+{
+	int retval;
+
+	Assert(out != NULL);
+	Assert(outlen != NULL);
+	Assert(in != NULL);
+
+	retval = SSL_select_next_proto((unsigned char **) out, outlen,
+				       alpn_protos, sizeof(alpn_protos),
+				       in, inlen);
+
+	if (*out == NULL || *outlen > sizeof(alpn_protos) || *outlen <= 0)
+		return SSL_TLSEXT_ERR_NOACK;
+
+	if (retval == OPENSSL_NPN_NEGOTIATED) {
+		return SSL_TLSEXT_ERR_OK;
+	} else {
+		/*
+		 * The client doesn't support our protocol.  Reject the connection
+		 * with TLS "no_application_protocol" alert, per RFC 7301.
+		 */
+		return SSL_TLSEXT_ERR_ALERT_FATAL;
+	}
+}
+
+int tls_configure_server(struct tls *ctx)
 {
 	EC_KEY *ecdh_key;
 	STACK_OF(X509_NAME) * cert_stack;
@@ -63,6 +101,7 @@ tls_configure_server(struct tls *ctx)
 		tls_set_errorx(ctx, "ssl context failure");
 		goto err;
 	}
+	SSL_CTX_set_alpn_select_cb(ctx->ssl_ctx, alpn_cb, NULL);
 
 	if (tls_configure_ssl(ctx) != 0)
 		goto err;
@@ -83,7 +122,7 @@ tls_configure_server(struct tls *ctx)
 		SSL_CTX_set_ecdh_auto(ctx->ssl_ctx, 1);
 	} else if (ctx->config->ecdhecurve != NID_undef) {
 		if ((ecdh_key = EC_KEY_new_by_curve_name(
-		    ctx->config->ecdhecurve)) == NULL) {
+			     ctx->config->ecdhecurve)) == NULL) {
 			tls_set_errorx(ctx, "failed to set ECDHE curve");
 			goto err;
 		}
@@ -92,9 +131,10 @@ tls_configure_server(struct tls *ctx)
 		EC_KEY_free(ecdh_key);
 	}
 
-	if (ctx->config->ciphers_server == 1)
+	if (ctx->config->ciphers_server == 1) {
 		SSL_CTX_set_options(ctx->ssl_ctx,
-		    SSL_OP_CIPHER_SERVER_PREFERENCE);
+				    SSL_OP_CIPHER_SERVER_PREFERENCE);
+	}
 
 	if (SSL_CTX_set_tlsext_status_cb(ctx->ssl_ctx, tls_ocsp_stapling_callback) != 1) {
 		tls_set_errorx(ctx, "ssl OCSP stapling setup failure");
@@ -116,22 +156,20 @@ tls_configure_server(struct tls *ctx)
 	}
 
 	cert_stack = SSL_load_client_CA_file(ctx->config->ca_file);
-    	SSL_CTX_set_client_CA_list(ctx->ssl_ctx, cert_stack);
+	SSL_CTX_set_client_CA_list(ctx->ssl_ctx, cert_stack);
 
 	return (0);
 
- err:
+err:
 	return (-1);
 }
 
-int
-tls_accept_socket(struct tls *ctx, struct tls **cctx, int socket)
+int tls_accept_socket(struct tls *ctx, struct tls **cctx, int socket)
 {
 	return (tls_accept_fds(ctx, cctx, socket, socket));
 }
 
-int
-tls_accept_fds(struct tls *ctx, struct tls **cctx, int fd_read, int fd_write)
+int tls_accept_fds(struct tls *ctx, struct tls **cctx, int fd_read, int fd_write)
 {
 	struct tls *conn_ctx = NULL;
 
@@ -163,7 +201,7 @@ tls_accept_fds(struct tls *ctx, struct tls **cctx, int fd_read, int fd_write)
 
 	return (0);
 
- err:
+err:
 	usual_tls_free(conn_ctx);
 
 	*cctx = NULL;
@@ -171,8 +209,7 @@ tls_accept_fds(struct tls *ctx, struct tls **cctx, int fd_read, int fd_write)
 	return (-1);
 }
 
-int
-tls_handshake_server(struct tls *ctx)
+int tls_handshake_server(struct tls *ctx)
 {
 	int ssl_ret;
 	int rv = -1;
@@ -191,7 +228,7 @@ tls_handshake_server(struct tls *ctx)
 	ctx->state |= TLS_HANDSHAKE_COMPLETE;
 	rv = 0;
 
- err:
+err:
 	return (rv);
 }
 
