@@ -5,6 +5,7 @@ import filelock
 import pytest
 
 from .utils import (
+    LDAP_SUPPORT,
     LINUX,
     LONG_PASSWORD,
     PG_SUPPORTS_SCRAM,
@@ -12,7 +13,9 @@ from .utils import (
     TLS_SUPPORT,
     USE_SUDO,
     Bouncer,
+    OpenLDAP,
     Postgres,
+    Proxy,
     run,
     sudo,
 )
@@ -128,6 +131,9 @@ def pg(tmp_path_factory, cert_dir):
     for i in range(8):
         pg.sql(f"create database p{i}")
 
+    if LDAP_SUPPORT:
+        pg.sql("create user ldapuser1 login password 'secret1'")
+
     pg.sql("create database unconfigured_auth_database")
     pg.sql("create user bouncer")
 
@@ -141,6 +147,7 @@ def pg(tmp_path_factory, cert_dir):
     pg.sql("create user maxedout5;")
     pg.sql("create user poolsize1;")
     pg.sql("create user respoolsize1;")
+    pg.sql("create user clientstate;")
     pg.sql("create user test_error_message_user;")
     pg.sql(f"create user longpass with password '{LONG_PASSWORD}';")
     pg.sql("create user stats password 'stats';")
@@ -172,7 +179,18 @@ def pg(tmp_path_factory, cert_dir):
     pg.cleanup()
 
 
-@pytest.mark.asyncio
+@pytest.fixture
+async def proxy(pg, tmp_path):
+    """Starts a new proxy process"""
+    proxy = Proxy(pg)
+
+    proxy.start()
+
+    yield proxy
+
+    proxy.cleanup()
+
+
 @pytest.fixture
 async def bouncer(pg, tmp_path):
     """Starts a new PgBouncer process"""
@@ -185,6 +203,22 @@ async def bouncer(pg, tmp_path):
     await bouncer.cleanup()
 
 
+@pytest.fixture
+async def bouncer_with_openldap(pg, tmp_path, monkeypatch):
+    """Starts a new PgBouncer process"""
+    bouncer = Bouncer(pg, tmp_path / "bouncer")
+    ldap = OpenLDAP(tmp_path)
+    bouncer.ldap = ldap
+    monkeypatch.setenv("LDAPCONF", str(tmp_path / "ldap/ldap.conf"))
+    ldap.startup()
+    await bouncer.start()
+
+    yield bouncer
+
+    ldap.cleanup()
+    await bouncer.cleanup()
+
+
 @pytest.fixture(autouse=True)
 def pg_log(pg):
     """Prints the Postgres logs that were created during the test
