@@ -8,6 +8,7 @@ import pytest
 
 from .utils import (
     FREEBSD,
+    LDAP_SUPPORT,
     LONG_PASSWORD,
     MACOS,
     PG_SUPPORTS_SCRAM,
@@ -309,6 +310,19 @@ def test_scram_both(bouncer):
     bouncer.test(dbname="p62", user="scramuser1", password="foo")
 
 
+@pytest.mark.skipif("not PG_SUPPORTS_SCRAM")
+def test_scram_both_reauthentication_using_cache(bouncer):
+    bouncer.admin(f"set auth_type='scram-sha-256'")
+
+    # Make multiple connections to exercise the SCRAM secrets cache logic
+    for _ in range(1, 10):
+        # plain-text password in userlist.txt
+        bouncer.test(dbname="p61", user="scramuser3", password="baz")
+
+        # SCRAM password in userlist.txt
+        bouncer.test(dbname="p62", user="scramuser1", password="foo")
+
+
 @pytest.mark.skipif("WINDOWS", reason="Windows does not have SIGHUP")
 def test_auth_dbname_usage(
     bouncer,
@@ -394,8 +408,9 @@ def test_auth_dbname_usage_global_setting(
     with bouncer.log_contains(
         'cannot use the reserved "pgbouncer" database as an auth_dbname', 1
     ):
-        with bouncer.run_with_config(config):
-            pass
+        with pytest.raises(psycopg.DatabaseError):
+            with bouncer.run_with_config(config):
+                pass
 
 
 @pytest.mark.skipif("WINDOWS", reason="Windows does not have SIGHUP")
@@ -1111,3 +1126,158 @@ def test_auth_user_at_db_level_with_same_forced_user(bouncer):
         with bouncer.conn(dbname="p3", user="postgres", password="asdasd") as cn:
             with cn.cursor() as cur:
                 cur.execute("select 1")
+
+
+@pytest.mark.skipif("WINDOWS", reason="We do not expect to support ldap on Windows")
+@pytest.mark.skipif(not LDAP_SUPPORT, reason="pgbouncer is built without LDAP support")
+def test_ldap_auth(bouncer_with_openldap):
+    openldap = bouncer_with_openldap.ldap
+    # 1 test "simple bind"
+    hba_conf_file = bouncer_with_openldap.config_dir / "ldap_hba.conf"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            "host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 "
+            f'ldapport={openldap.ldap_port} ldapprefix="uid=" '
+            f'ldapsuffix=",dc=example,dc=net"\n'
+        )
+    bouncer_with_openldap.write_ini(f"auth_type = hba")
+    bouncer_with_openldap.write_ini(f"auth_hba_file = {hba_conf_file}")
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 2 test "search+bind"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f'host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 ldapport={openldap.ldap_port} ldapbasedn="dc=example,dc=net"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 3 test "multiple servers"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f'host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 ldapport={openldap.ldap_port} ldapbasedn="dc=example,dc=net"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 4 test "LDAP URLs"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f'host all ldapuser1 0.0.0.0/0 ldap ldapurl="ldap://127.0.0.1:{openldap.ldap_port}/dc=example,dc=net?uid?sub"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 5 test "ldapsearchattribute"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 ldapport={openldap.ldap_port} "
+            f'ldapbasedn="dc=example,dc=net" ldapsearchattribute=uid'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 6 test "search filters"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 ldapport={openldap.ldap_port} "
+            f'ldapbasedn="dc=example,dc=net" ldapsearchfilter="uid=$username"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 7 test "search filters in LDAP URLs"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap "
+            f'ldapurl="ldap://127.0.0.1:{openldap.ldap_port}/dc=example,dc=net??sub?(|(uid=$username)(mail=$username))"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8 test ldaps
+    # 8.1 test "search filters in LDAP URLs"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap "
+            f'ldapurl="ldaps://127.0.0.1:{openldap.ldaps_port}/dc=example,dc=net??sub?(|(uid=$username)(mail=$username))"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.2 test "simple bind"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            "host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 "
+            f'ldapport={openldap.ldap_port} ldapprefix="uid=" '
+            f'ldapsuffix=",dc=example,dc=net" ldaptls=1\n'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.3 test "search+bind"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f'host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 ldapport={openldap.ldap_port} ldapbasedn="dc=example,dc=net" ldaptls=1'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.4 test "LDAP URLs"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f'host all ldapuser1 0.0.0.0/0 ldap ldapurl="ldaps://127.0.0.1:{openldap.ldaps_port}/dc=example,dc=net?uid?sub"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.5 test "search filters in LDAP URLs"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 ldapport={openldap.ldap_port} "
+            f'ldapbasedn="dc=example,dc=net" ldapsearchfilter="uid=$username" ldaptls=1'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.6 test ldaps with normal ldap port, connection failed
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap "
+            f'ldapurl="ldaps://127.0.0.1:{openldap.ldap_port}/dc=example,dc=net??sub?(|(uid=$username)(mail=$username))"'
+        )
+    bouncer_with_openldap.admin("reload")
+    with pytest.raises(psycopg.OperationalError, match="connection failed"):
+        bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.7 test ldap with ldaptls=1 with ldap ssl port, connection failed, compared with 8.2
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            "host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 "
+            f'ldapport={openldap.ldaps_port} ldapprefix="uid=" '
+            f'ldapsuffix=",dc=example,dc=net" ldaptls=1\n'
+        )
+    bouncer_with_openldap.admin("reload")
+    with pytest.raises(psycopg.OperationalError, match="connection failed"):
+        bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.8 test ldap with ldaps:// and ldaptls=1 both set with ldap port, connection failed
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap "
+            f'ldapurl="ldaps://127.0.0.1:{openldap.ldaps_port}/dc=example,dc=net??sub?(|(uid=$username)(mail=$username))" ldaptls=1'
+        )
+    bouncer_with_openldap.admin("reload")
+    with pytest.raises(psycopg.OperationalError, match="connection failed"):
+        bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 8.9 test ldap with ldaps:// and ldaptls=1 both set with ldap ssl port, connection failed
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f"host all ldapuser1 0.0.0.0/0 ldap "
+            f'ldapurl="ldaps://127.0.0.1:{openldap.ldap_port}/dc=example,dc=net??sub?(|(uid=$username)(mail=$username))" ldaptls=1'
+        )
+    bouncer_with_openldap.admin("reload")
+    with pytest.raises(psycopg.OperationalError, match="connection failed"):
+        bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 9 test "hba format"
+    with open(hba_conf_file, "w") as f:
+        f.write(
+            f'host all ldapuser1 0.0.0.0/0 ldap ldapserver=127.0.0.1 "ldapport"={openldap.ldap_port} '
+            f'ldapbasedn="dc=example,dc=net" ,ldapsearchfilter="uid=$username"'
+        )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
+    # 10 test ldap auth_type
+    bouncer_with_openldap.write_ini(f"auth_type = ldap")
+    bouncer_with_openldap.write_ini(
+        f'auth_ldap_options = ldapurl="ldap://127.0.0.1:{openldap.ldap_port}/dc=example,dc=net?uid?sub"'
+    )
+    bouncer_with_openldap.admin("reload")
+    bouncer_with_openldap.test(user="ldapuser1", password="secret1")
